#' Find marker genes using scran.
#'
#' The wraps the `scran::findMarkers` function.
#'
#' @param sce A SingleCellExperiment object
#' @param pars parameters passed to the function
#'
#' @return A list containing the time of the computation and results of the
#'  method.
#'
run_scran <- function(sce, pars) {

  times <- system.time({
    raw_result <- scran::findMarkers(
      sce,
      test.type = pars$test.type,
      pval.type = pars$pval.type)
  })

  result <- lapply(raw_result, function(x) {

    x <- as.data.frame(x)
    x <- tibble::rownames_to_column(x, var = "gene")
    x <- tibble::as_tibble(x)

    if (pars$test.type == "wilcox") {
        raw_statistic_col <- "summary.AUC"
    } else {
        raw_statistic_col <- "summary.logFC"
    }

    if (pars$pval.type == "any") {

      # We keep `Top` - which doesn't fit into any category.
      x <- dplyr::select(
        x,
        p_value = p.value,
        raw_statistic = !!raw_statistic_col,
        p_value_adj = FDR,
        gene = gene,
        top = Top
      )
    } else {
      x <- dplyr::select(
        x,
        p_value = p.value,
        raw_statistic = !!raw_statistic_col,
        p_value_adj = FDR,
        gene = gene
      )

    }

    # We do know the log fold change when not using the Wilcox test, but it
    # doesn't add any extra information.
    x <- dplyr::mutate(
      x,
      log_fc = 0,
      scaled_statistic = 0
    )

    x
  })

  cluster_names <- names(result)
  for (i in seq_along(result)) {
    result[[i]] <- dplyr::mutate(result[[i]], cluster = cluster_names[[i]])
  }
  result <- dplyr::bind_rows(!!!result)

  validate_mgs_result(result)

  # Get the elapsed time.
  time <- times[[3]]

  list(time = time, result = result, raw_result = raw_result, pars = pars)
}


